// Copyright 1997 by Patrik Simons
// This software is provided as is, no warranty of any kind is given.
#ifndef SMODELS_H
#define SMODELS_H

#include "program.h"
#include "dcl.h"
#include "stack.h"

class Atom;
class OptimizeRule;

class Smodels
{
public:
  Smodels ();
  virtual ~Smodels ();
  void init ();

  void setToBTrue (Atom *a);
  void setToBFalse (Atom *a);
  void pick ();
  void expand ();
  void setup ();
  void revert ();
  bool conflict ();
  bool covered ();
  void set_conflict ();
  Atom *unwind ();
  void unwind_all ();
  void unwind_to_setup ();
  Atom *backtrack ();
  Atom *backjump ();
  void backtrack (bool jump);
  int smodels (bool look, bool jump);
  int smodels (bool look, bool jump, long max_conflicts);
  int model (bool look = true, bool jump = false);
  int wellfounded ();
  bool testPos (Atom *);
  bool testNeg (Atom *);
  void testScore (long pos, long neg, long i, long &hiscore1,
		 long &hiscore2, long &hi, int &positive);
  void lookahead ();
  void resetDependency ();
  void shuffle ();
  void print ();
  void printAnswer ();
  void printStack ();
  void printRule (Rule *);

  void removeAtom (long);
  void addAtom (Atom *);

  Dcl dcl;
  Program program;
  Atom **atom;           // The atoms in the program. We walk the
			 // array when we do lookahead or compute the
			 // heuristic.
  long atomtop;          // Atoms on and above atomtop are in B and
			 // can be ignored.
  long score;            // Heuristics...
  Stack depends;         // Holds atoms with dependson{True,False} true
  Stack stack;           // Holds the backtrack information (the atoms
			 // in B). When full we have a stable model.
  long setup_top;        // Top of stack after setup
  long guesses;          // The number of atoms on the stack that are
			 // choice points. If its zero and we try to
			 // backtrack, then we have exhausted the
			 // search space and we should --
  bool fail;             // fail.
  bool conflict_found;   // If we have found a conflict this is true
  long max_models;       // The number of models we compute before we
			 // stop.
  long level;            // Mustn't backjump below this stack level.
  long lasti;            // Speed up lookahead by starting where we
			 // last stopped doing lookahead.
  Atom *cnflct;          // Backjump...
  bool sloppy_heuristic; // Don't compute full heuristic after lookahead
  OptimizeRule *optimizerule;

  long answer_number;
  long number_of_choice_points;
  long number_of_wrong_choices;
  long number_of_backjumps;
  long number_of_picked_atoms;
  long number_of_forced_atoms;
  unsigned long number_of_assignments;
};

#endif
